.checkboxes-list,
.radio-items-list {
  align-self: stretch;
  color: var(--text-secondary);
  display: flex;
  flex-direction: column;
  font-size: var(--text-size-02);
  gap: var(--spacing-02);
  letter-spacing: var(--letter-spacing-body-ui-02);
  line-height: var(--text-size-04);
}

.checkboxes-list label,
.radio-items-list label {
  align-items: center;
  align-self: stretch;
  display: flex !important;
  gap: var(--spacing-01);
  height: var(--spacing-05);
}

.input-container {
  display: flex;
  flex-direction: column;
  gap: var(--spacing-03);
  width: 100%;
}
