.rc-slider {
  flex: auto;
  margin: 4px -12px 0;
}

.rc-slider-rail,
.rc-slider-track {
  background-color: var(--slider-background-color);
  height: 2px;
}

.rc-slider-track {
  background-color: var(--fill-on-active);
}

.rc-slider-dot {
  background-color: var(--slider-background-color);
  border: var(--field-enabled);
  bottom: 0;
  height: 6px;
  width: 6px;
}

.rc-slider-dot-active {
  background-color: var(--fill-on-active);
}

.rc-slider-handle,
.rc-slider-handle:hover,
.rc-slider-handle-dragging.rc-slider-handle-dragging.rc-slider-handle-dragging,
.rc-slider-handle-click-focused:focus {
  background-color: var(--surfaces-bg-01);
  border: solid 6px var(--fill-on-active);
  border-color: var(--fill-on-active);
  border-radius: 100%;
  height: 16px;
  margin: -8px 0 0;
  position: absolute;
  width: 16px;
}

.rc-slider-handle:hover,
.rc-slider-handle-dragging.rc-slider-handle-dragging.rc-slider-handle-dragging {
  border: solid 4px var(--fill-on-active);
  box-shadow: unset;
  cursor: context-menu;
}

.rc-slider-handle-click-focused:focus {
  box-shadow: unset;
}

.rc-slider-handle:active {
  border: solid 6px var(--fill-on-active);
  box-shadow: unset;
  cursor: grabbing;
  height: 16px;
  margin-top: -8px;
  width: 16px;
}

.rc-slider-mark-text {
  color: var(--text-secondary);
  font-size: var(--text-size-01);
  line-height: var(--text-size-02);
  margin-top: var(--spacing-01);
  min-width: 44px;
}

.rc-slider-mark-text-active {
  color: var(--fill-on-active);
}

.slider-track-with-marks,
.slider-track-without-marks {
  display: flex;
  margin: 0;
  width: 100%;
}

.slider-track-without-marks {
  padding: 4px 24px !important;
}

.slider-track-with-marks {
  padding: 4px 24px 24px !important;
}

input.dash-input:invalid {
  outline: none;
}

/* Slider Label + Input Container ---------------------- */
.slider-label-input {
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: space-between;
  line-height: var(--text-size-03);
}

.slider-label-input label {
  width: 100%;
}

/* Slider Text Input ---------------------- */
.slider-text-input-container {
  display: flex;
  flex-direction: row;
  gap: var(--spacing-01);
}

.slider-text-input-field {
  background-color: transparent;
  color: var(--text-secondary);
  font-size: var(--text-size-02);
  max-width: 80px; /* required for Mozilla */
  padding: 0;
  text-align: center;
  text-decoration: underline;
  text-decoration-color: var(--border-subtle-alpha-01);
}

.slider-text-input-range-separator {
  color: var(--text-secondary);
  font-size: var(--text-size-02);
}

/* To remove number input spin box */
.slider-text-input-container input::-webkit-outer-spin-button,
input::-webkit-inner-spin-button {
  appearance: none;
  margin: 0;
}

.slider-text-input-container input[type="number"] {
  appearance: textfield;
}
