#logo {
  height: 40px;
  margin-left: -8px;
}

#container-id .grid-layout,
#container-id-2 .grid-layout,
#container-id-3 .grid-layout {
  display: flex;
  flex-direction: column;
  gap: 24px !important;
  width: 40%;
}

#container-id-2 .grid-layout,
#container-id-3 .grid-layout {
  width: 70%;
}
